//
//  View.m
//  Map
//
//  Created by NYU User on 10/11/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "View.h"
#import "Pin.h"


@implementation View


- (id) initWithFrame: (CGRect) frame {
	if ((self = [super initWithFrame: frame]) != nil) {
		
		// Initialization code
		
		pinPlaced = NO;
		
		aview = [AnnotationView alloc];
		
		manager = [[CLLocationManager alloc] init];
		manager.delegate = self;
		manager.desiredAccuracy = kCLLocationAccuracyBest;	//default; uses power
		[manager startUpdatingLocation];
		
		CGRect f = CGRectMake(0.0, 0.0, frame.size.width, frame.size.height);
		view = [[MKMapView alloc] initWithFrame: f];
		view.mapType = MKMapTypeHybrid;	//default is MKMapTypeStandard
		view.zoomEnabled = YES;		//default; double tap and pinch
		view.delegate = self;
		view.autoresizesSubviews = YES;
		//Latitude and longitude of the four corners of NYU campus:
		
		CLLocationCoordinate2D coordinates[] = {
			{40.728804, -73.99500},		//West 4th St. & Mercer St.
			{40.730978, -73.999625},	//West 4th St. & MacDougal St.
			{40.730438, -74.000092},	//West 3rd St. & MacDougal St.
			{40.728202, -73.99555}		//West 3rd St. & Mercer St.
		};
		const size_t count = sizeof coordinates / sizeof coordinates[0];
		
		overlay = [MKPolygon polygonWithCoordinates: coordinates count: count];
		
		CGRect rect;
		
		rect.origin.x = -self.center.x;
		rect.origin.y = -self.center.y;
		
		rect.size.width = self.bounds.size.width*3;
		rect.size.height = self.bounds.size.height*3;
		
		accView = [[AccuracyView alloc] initWithFrame:rect];
	
		
		[view addSubview:accView];
		[self addSubview:view];
		[view addOverlay:overlay];
		
		//view.hidden = YES;
		
	}
	return self;
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

- (void) dealloc {
	[geocoder release];
	[view release];
	[aview release];
	[accView release];
	[overlay release];
	[manager release];
	[super dealloc];
}

#pragma mark -
#pragma mark method of protocol CLLocationManagerDelegate

- (void) locationManager: (CLLocationManager *) m
	 didUpdateToLocation: (CLLocation *) newLocation
			fromLocation: (CLLocation *) oldLocation {
	
	if (m == manager) {
		CLLocationDistance meters = 100.0;
		
		newLocation = [[CLLocation alloc] initWithLatitude: 40.7285808 longitude: -73.9911222];
		
		view.region = MKCoordinateRegionMakeWithDistance(
			newLocation.coordinate,
			meters * self.bounds.size.height / self.bounds.size.width, //north-south
			meters		//east-west
		);
		
		
		[manager stopUpdatingLocation];
		
		//A reverse geocoder changes latitude/longitude
		//into a street address.
		geocoder = [[MKReverseGeocoder alloc]
					initWithCoordinate: newLocation.coordinate];
		geocoder.delegate = self;
		[geocoder start];
		[newLocation release];
	}
}

#pragma mark -
#pragma mark methods of protocol MKReverseGeocoderDelegate

//The placemark holds the street address found by the reverse geocoder.

- (void) reverseGeocoder: (MKReverseGeocoder *) g
		didFindPlacemark: (MKPlacemark *) placemark {
	
	if (g == geocoder) {
		CLLocationCoordinate2D c = geocoder.coordinate;
		NSString *subtitle = [NSString
			stringWithFormat: @"%g°%c %g°%c %@ County",
			fabs(c.latitude),
			c.latitude >= 0 ? 'N' : 'S',
			fabs(c.longitude),
			c.longitude >= 0 ? 'E' : 'W',
			placemark.locality
		];
		
		Pin *p = [[Pin alloc]
				initWithTitle: placemark.title
				subtitle: subtitle
				coordinate: geocoder.coordinate
		];
		
		
		
		[view addAnnotation: p];	//Tap on the red pin.
		[p release];
		[self drawAccuracyCircle];
		
	}
}

- (void) reverseGeocoder: (MKReverseGeocoder *) g
		didFailWithError: (NSError *) error {
	
	
			NSLog(@"geocoder failed %@", error);
			
	// Show coordinates as title if reverseGeocoder fails to come up with a placemark
	
			if (g == geocoder) {
				
				CLLocationCoordinate2D c = geocoder.coordinate;
		
				Pin *p = [[Pin alloc]
				  initWithTitle: [NSString
					  stringWithFormat: @"%g°%c %g°%c",
					  fabs(c.latitude),
					  c.latitude >= 0 ? 'N' : 'S',
					  fabs(c.longitude),
					  c.longitude >= 0 ? 'E' : 'W'
				  ]
				  subtitle: nil
				  coordinate: geocoder.coordinate
				];
				
				[view addAnnotation: p];	//Tap on the red pin.
				[p release];
			}
}

- (void)drawAccuracyCircle
{
	if(pinPlaced)
	{
		float accuracy = manager.location.horizontalAccuracy;
		
		CGRect rectangle;
		
		Pin *pin = [view.annotations objectAtIndex:0];
		
		float spanx = view.region.span.longitudeDelta; // longitudinal span of the map being displayed
		float deltaX = view.region.center.longitude - pin.coordinate.longitude;
		float deltaY = view.region.center.latitude - pin.coordinate.latitude;
		
		accuracy /= 111120; // 1 degree of the earth's arc is equal to 111120 meters;
		
		rectangle.size.width = accuracy / spanx * self.bounds.size.width;
		rectangle.size.height = rectangle.size.width;
		rectangle.origin.x = (0.5 - deltaX/spanx)*self.bounds.size.width-rectangle.size.width/2;
		rectangle.origin.y = (0.5 + deltaY/spanx)*self.bounds.size.height-rectangle.size.height/2;
		
		// don't display accuracy circle if zoomed out enough so pin covers entire circle
		if(rectangle.size.width < aview.image.size.width)
		{
			rectangle.size.width = 0;
			rectangle.size.height = 0;
		}
		
		
		[accView setFrame:rectangle];
		[accView setNeedsDisplayInRect:rectangle];
	}
}
	

#pragma mark -
#pragma mark methods of protocol MKMapViewDelegate

- (void)mapViewDidFinishLoadingMap:(MKMapView *)mapView {
	NSLog(@"Google finally got its act together and decided it was done sending data!");
	//Redraw accuracy circle whenever map changes
	[self drawAccuracyCircle];
}	

- (void)mapView:(MKMapView *)mapView regionDidChangeAnimated:(BOOL)animated {
	NSLog(@"Region changed");
	[self drawAccuracyCircle];
}


- (MKAnnotationView *) mapView: (MKMapView *) v
			 viewForAnnotation: (id <MKAnnotation>) annotation {
	
	
	[aview initWithAnnotation:annotation reuseIdentifier:nil];
	
	aview.canShowCallout = YES;

	/*
	aview.image = [UIImage imageNamed: @"nyu2.png"];
	aview.image = [UIImage imageWithCGImage:aview.image.CGImage scale:2 orientation:UIImageOrientationUp];
	*/
	pinPlaced = YES;
	
	return aview;
}

- (MKOverlayView *) mapView: (MKMapView *) v
			 viewForOverlay: (id <MKOverlay>) polygon {
	
	
	MKPolygonView *mView = [[MKPolygonView alloc] initWithPolygon: overlay];
	mView.fillColor = [UIColor
					   colorWithRed: 82 / 255.0
					   green: 46 / 255.0
					   blue: 145 / 255.0
					   alpha: .5];
	return mView;
}

@end